@@ -70,7 +70,8 @@ _io__TextIOBase_read(PyObject *self, PyTypeObject *cls, PyObject *const *args, P
     PyObject *argsbuf[1];
     int size = -1;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 0, 1, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 0, /*maxpos*/ 1, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -124,7 +125,8 @@ _io__TextIOBase_readline(PyObject *self, PyTypeObject *cls, PyObject *const *arg
     PyObject *argsbuf[1];
     int size = -1;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 0, 1, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 0, /*maxpos*/ 1, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -178,7 +180,8 @@ _io__TextIOBase_write(PyObject *self, PyTypeObject *cls, PyObject *const *args,
     PyObject *argsbuf[1];
     const char *s;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 1, 1, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 1, /*maxpos*/ 1, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -315,9 +318,11 @@ _io_IncrementalNewlineDecoder___init__(PyObject *self, PyObject *args, PyObject
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(decoder), &_Py_ID(translate), &_Py_ID(errors), },
     };
     #undef NUM_KEYWORDS
@@ -342,7 +347,8 @@ _io_IncrementalNewlineDecoder___init__(PyObject *self, PyObject *args, PyObject
     int translate;
     PyObject *errors = NULL;
 
-    fastargs = _PyArg_UnpackKeywords(_PyTuple_CAST(args)->ob_item, nargs, kwargs, NULL, &_parser, 2, 3, 0, argsbuf);
+    fastargs = _PyArg_UnpackKeywords(_PyTuple_CAST(args)->ob_item, nargs, kwargs, NULL, &_parser,
+            /*minpos*/ 2, /*maxpos*/ 3, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!fastargs) {
         goto exit;
     }
@@ -375,7 +381,7 @@ _io_IncrementalNewlineDecoder_decode_impl(nldecoder_object *self,
                                           PyObject *input, int final);
 
 static PyObject *
-_io_IncrementalNewlineDecoder_decode(nldecoder_object *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+_io_IncrementalNewlineDecoder_decode(PyObject *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -384,9 +390,11 @@ _io_IncrementalNewlineDecoder_decode(nldecoder_object *self, PyObject *const *ar
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(input), &_Py_ID(final), },
     };
     #undef NUM_KEYWORDS
@@ -408,7 +416,8 @@ _io_IncrementalNewlineDecoder_decode(nldecoder_object *self, PyObject *const *ar
     PyObject *input;
     int final = 0;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 1, 2, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 1, /*maxpos*/ 2, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -421,7 +430,7 @@ _io_IncrementalNewlineDecoder_decode(nldecoder_object *self, PyObject *const *ar
         goto exit;
     }
 skip_optional_pos:
-    return_value = _io_IncrementalNewlineDecoder_decode_impl(self, input, final);
+    return_value = _io_IncrementalNewlineDecoder_decode_impl((nldecoder_object *)self, input, final);
 
 exit:
     return return_value;
@@ -439,9 +448,9 @@ static PyObject *
 _io_IncrementalNewlineDecoder_getstate_impl(nldecoder_object *self);
 
 static PyObject *
-_io_IncrementalNewlineDecoder_getstate(nldecoder_object *self, PyObject *Py_UNUSED(ignored))
+_io_IncrementalNewlineDecoder_getstate(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
-    return _io_IncrementalNewlineDecoder_getstate_impl(self);
+    return _io_IncrementalNewlineDecoder_getstate_impl((nldecoder_object *)self);
 }
 
 PyDoc_STRVAR(_io_IncrementalNewlineDecoder_setstate__doc__,
@@ -452,6 +461,20 @@ PyDoc_STRVAR(_io_IncrementalNewlineDecoder_setstate__doc__,
 #define _IO_INCREMENTALNEWLINEDECODER_SETSTATE_METHODDEF    \
     {"setstate", (PyCFunction)_io_IncrementalNewlineDecoder_setstate, METH_O, _io_IncrementalNewlineDecoder_setstate__doc__},
 
+static PyObject *
+_io_IncrementalNewlineDecoder_setstate_impl(nldecoder_object *self,
+                                            PyObject *state);
+
+static PyObject *
+_io_IncrementalNewlineDecoder_setstate(PyObject *self, PyObject *state)
+{
+    PyObject *return_value = NULL;
+
+    return_value = _io_IncrementalNewlineDecoder_setstate_impl((nldecoder_object *)self, state);
+
+    return return_value;
+}
+
 PyDoc_STRVAR(_io_IncrementalNewlineDecoder_reset__doc__,
 "reset($self, /)\n"
 "--\n"
@@ -464,9 +487,9 @@ static PyObject *
 _io_IncrementalNewlineDecoder_reset_impl(nldecoder_object *self);
 
 static PyObject *
-_io_IncrementalNewlineDecoder_reset(nldecoder_object *self, PyObject *Py_UNUSED(ignored))
+_io_IncrementalNewlineDecoder_reset(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
-    return _io_IncrementalNewlineDecoder_reset_impl(self);
+    return _io_IncrementalNewlineDecoder_reset_impl((nldecoder_object *)self);
 }
 
 PyDoc_STRVAR(_io_TextIOWrapper___init____doc__,
@@ -519,9 +542,11 @@ _io_TextIOWrapper___init__(PyObject *self, PyObject *args, PyObject *kwargs)
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(buffer), &_Py_ID(encoding), &_Py_ID(errors), &_Py_ID(newline), &_Py_ID(line_buffering), &_Py_ID(write_through), },
     };
     #undef NUM_KEYWORDS
@@ -549,7 +574,8 @@ _io_TextIOWrapper___init__(PyObject *self, PyObject *args, PyObject *kwargs)
     int line_buffering = 0;
     int write_through = 0;
 
-    fastargs = _PyArg_UnpackKeywords(_PyTuple_CAST(args)->ob_item, nargs, kwargs, NULL, &_parser, 1, 6, 0, argsbuf);
+    fastargs = _PyArg_UnpackKeywords(_PyTuple_CAST(args)->ob_item, nargs, kwargs, NULL, &_parser,
+            /*minpos*/ 1, /*maxpos*/ 6, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!fastargs) {
         goto exit;
     }
@@ -648,7 +674,7 @@ _io_TextIOWrapper_reconfigure_impl(textio *self, PyObject *encoding,
                                    PyObject *write_through_obj);
 
 static PyObject *
-_io_TextIOWrapper_reconfigure(textio *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
+_io_TextIOWrapper_reconfigure(PyObject *self, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames)
 {
     PyObject *return_value = NULL;
     #if defined(Py_BUILD_CORE) && !defined(Py_BUILD_CORE_MODULE)
@@ -657,9 +683,11 @@ _io_TextIOWrapper_reconfigure(textio *self, PyObject *const *args, Py_ssize_t na
     static struct {
         PyGC_Head _this_is_not_used;
         PyObject_VAR_HEAD
+        Py_hash_t ob_hash;
         PyObject *ob_item[NUM_KEYWORDS];
     } _kwtuple = {
         .ob_base = PyVarObject_HEAD_INIT(&PyTuple_Type, NUM_KEYWORDS)
+        .ob_hash = -1,
         .ob_item = { &_Py_ID(encoding), &_Py_ID(errors), &_Py_ID(newline), &_Py_ID(line_buffering), &_Py_ID(write_through), },
     };
     #undef NUM_KEYWORDS
@@ -684,7 +712,8 @@ _io_TextIOWrapper_reconfigure(textio *self, PyObject *const *args, Py_ssize_t na
     PyObject *line_buffering_obj = Py_None;
     PyObject *write_through_obj = Py_None;
 
-    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser, 0, 0, 0, argsbuf);
+    args = _PyArg_UnpackKeywords(args, nargs, NULL, kwnames, &_parser,
+            /*minpos*/ 0, /*maxpos*/ 0, /*minkw*/ 0, /*varpos*/ 0, argsbuf);
     if (!args) {
         goto exit;
     }
@@ -718,7 +747,7 @@ _io_TextIOWrapper_reconfigure(textio *self, PyObject *const *args, Py_ssize_t na
     write_through_obj = args[4];
 skip_optional_kwonly:
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_reconfigure_impl(self, encoding, errors, newline_obj, line_buffering_obj, write_through_obj);
+    return_value = _io_TextIOWrapper_reconfigure_impl((textio *)self, encoding, errors, newline_obj, line_buffering_obj, write_through_obj);
     Py_END_CRITICAL_SECTION();
 
 exit:
@@ -737,12 +766,12 @@ static PyObject *
 _io_TextIOWrapper_detach_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper_detach(textio *self, PyObject *Py_UNUSED(ignored))
+_io_TextIOWrapper_detach(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_detach_impl(self);
+    return_value = _io_TextIOWrapper_detach_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -760,7 +789,7 @@ static PyObject *
 _io_TextIOWrapper_write_impl(textio *self, PyObject *text);
 
 static PyObject *
-_io_TextIOWrapper_write(textio *self, PyObject *arg)
+_io_TextIOWrapper_write(PyObject *self, PyObject *arg)
 {
     PyObject *return_value = NULL;
     PyObject *text;
@@ -771,7 +800,7 @@ _io_TextIOWrapper_write(textio *self, PyObject *arg)
     }
     text = arg;
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_write_impl(self, text);
+    return_value = _io_TextIOWrapper_write_impl((textio *)self, text);
     Py_END_CRITICAL_SECTION();
 
 exit:
@@ -790,7 +819,7 @@ static PyObject *
 _io_TextIOWrapper_read_impl(textio *self, Py_ssize_t n);
 
 static PyObject *
-_io_TextIOWrapper_read(textio *self, PyObject *const *args, Py_ssize_t nargs)
+_io_TextIOWrapper_read(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     Py_ssize_t n = -1;
@@ -806,7 +835,7 @@ _io_TextIOWrapper_read(textio *self, PyObject *const *args, Py_ssize_t nargs)
     }
 skip_optional:
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_read_impl(self, n);
+    return_value = _io_TextIOWrapper_read_impl((textio *)self, n);
     Py_END_CRITICAL_SECTION();
 
 exit:
@@ -825,7 +854,7 @@ static PyObject *
 _io_TextIOWrapper_readline_impl(textio *self, Py_ssize_t size);
 
 static PyObject *
-_io_TextIOWrapper_readline(textio *self, PyObject *const *args, Py_ssize_t nargs)
+_io_TextIOWrapper_readline(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     Py_ssize_t size = -1;
@@ -850,7 +879,7 @@ _io_TextIOWrapper_readline(textio *self, PyObject *const *args, Py_ssize_t nargs
     }
 skip_optional:
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_readline_impl(self, size);
+    return_value = _io_TextIOWrapper_readline_impl((textio *)self, size);
     Py_END_CRITICAL_SECTION();
 
 exit:
@@ -887,7 +916,7 @@ static PyObject *
 _io_TextIOWrapper_seek_impl(textio *self, PyObject *cookieObj, int whence);
 
 static PyObject *
-_io_TextIOWrapper_seek(textio *self, PyObject *const *args, Py_ssize_t nargs)
+_io_TextIOWrapper_seek(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     PyObject *cookieObj;
@@ -906,7 +935,7 @@ _io_TextIOWrapper_seek(textio *self, PyObject *const *args, Py_ssize_t nargs)
     }
 skip_optional:
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_seek_impl(self, cookieObj, whence);
+    return_value = _io_TextIOWrapper_seek_impl((textio *)self, cookieObj, whence);
     Py_END_CRITICAL_SECTION();
 
 exit:
@@ -929,12 +958,12 @@ static PyObject *
 _io_TextIOWrapper_tell_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper_tell(textio *self, PyObject *Py_UNUSED(ignored))
+_io_TextIOWrapper_tell(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_tell_impl(self);
+    return_value = _io_TextIOWrapper_tell_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -952,7 +981,7 @@ static PyObject *
 _io_TextIOWrapper_truncate_impl(textio *self, PyObject *pos);
 
 static PyObject *
-_io_TextIOWrapper_truncate(textio *self, PyObject *const *args, Py_ssize_t nargs)
+_io_TextIOWrapper_truncate(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     PyObject *pos = Py_None;
@@ -966,7 +995,7 @@ _io_TextIOWrapper_truncate(textio *self, PyObject *const *args, Py_ssize_t nargs
     pos = args[0];
 skip_optional:
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_truncate_impl(self, pos);
+    return_value = _io_TextIOWrapper_truncate_impl((textio *)self, pos);
     Py_END_CRITICAL_SECTION();
 
 exit:
@@ -985,12 +1014,12 @@ static PyObject *
 _io_TextIOWrapper_fileno_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper_fileno(textio *self, PyObject *Py_UNUSED(ignored))
+_io_TextIOWrapper_fileno(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_fileno_impl(self);
+    return_value = _io_TextIOWrapper_fileno_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -1008,12 +1037,12 @@ static PyObject *
 _io_TextIOWrapper_seekable_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper_seekable(textio *self, PyObject *Py_UNUSED(ignored))
+_io_TextIOWrapper_seekable(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_seekable_impl(self);
+    return_value = _io_TextIOWrapper_seekable_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -1031,12 +1060,12 @@ static PyObject *
 _io_TextIOWrapper_readable_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper_readable(textio *self, PyObject *Py_UNUSED(ignored))
+_io_TextIOWrapper_readable(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_readable_impl(self);
+    return_value = _io_TextIOWrapper_readable_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -1054,12 +1083,12 @@ static PyObject *
 _io_TextIOWrapper_writable_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper_writable(textio *self, PyObject *Py_UNUSED(ignored))
+_io_TextIOWrapper_writable(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_writable_impl(self);
+    return_value = _io_TextIOWrapper_writable_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -1077,12 +1106,12 @@ static PyObject *
 _io_TextIOWrapper_isatty_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper_isatty(textio *self, PyObject *Py_UNUSED(ignored))
+_io_TextIOWrapper_isatty(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_isatty_impl(self);
+    return_value = _io_TextIOWrapper_isatty_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -1100,12 +1129,12 @@ static PyObject *
 _io_TextIOWrapper_flush_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper_flush(textio *self, PyObject *Py_UNUSED(ignored))
+_io_TextIOWrapper_flush(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_flush_impl(self);
+    return_value = _io_TextIOWrapper_flush_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -1123,12 +1152,12 @@ static PyObject *
 _io_TextIOWrapper_close_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper_close(textio *self, PyObject *Py_UNUSED(ignored))
+_io_TextIOWrapper_close(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_close_impl(self);
+    return_value = _io_TextIOWrapper_close_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -1148,12 +1177,12 @@ static PyObject *
 _io_TextIOWrapper_name_get_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper_name_get(textio *self, void *Py_UNUSED(context))
+_io_TextIOWrapper_name_get(PyObject *self, void *Py_UNUSED(context))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_name_get_impl(self);
+    return_value = _io_TextIOWrapper_name_get_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -1173,12 +1202,12 @@ static PyObject *
 _io_TextIOWrapper_closed_get_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper_closed_get(textio *self, void *Py_UNUSED(context))
+_io_TextIOWrapper_closed_get(PyObject *self, void *Py_UNUSED(context))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_closed_get_impl(self);
+    return_value = _io_TextIOWrapper_closed_get_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -1198,12 +1227,12 @@ static PyObject *
 _io_TextIOWrapper_newlines_get_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper_newlines_get(textio *self, void *Py_UNUSED(context))
+_io_TextIOWrapper_newlines_get(PyObject *self, void *Py_UNUSED(context))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_newlines_get_impl(self);
+    return_value = _io_TextIOWrapper_newlines_get_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -1223,12 +1252,12 @@ static PyObject *
 _io_TextIOWrapper_errors_get_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper_errors_get(textio *self, void *Py_UNUSED(context))
+_io_TextIOWrapper_errors_get(PyObject *self, void *Py_UNUSED(context))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper_errors_get_impl(self);
+    return_value = _io_TextIOWrapper_errors_get_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -1248,12 +1277,12 @@ static PyObject *
 _io_TextIOWrapper__CHUNK_SIZE_get_impl(textio *self);
 
 static PyObject *
-_io_TextIOWrapper__CHUNK_SIZE_get(textio *self, void *Py_UNUSED(context))
+_io_TextIOWrapper__CHUNK_SIZE_get(PyObject *self, void *Py_UNUSED(context))
 {
     PyObject *return_value = NULL;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper__CHUNK_SIZE_get_impl(self);
+    return_value = _io_TextIOWrapper__CHUNK_SIZE_get_impl((textio *)self);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
@@ -1273,14 +1302,14 @@ static int
 _io_TextIOWrapper__CHUNK_SIZE_set_impl(textio *self, PyObject *value);
 
 static int
-_io_TextIOWrapper__CHUNK_SIZE_set(textio *self, PyObject *value, void *Py_UNUSED(context))
+_io_TextIOWrapper__CHUNK_SIZE_set(PyObject *self, PyObject *value, void *Py_UNUSED(context))
 {
     int return_value;
 
     Py_BEGIN_CRITICAL_SECTION(self);
-    return_value = _io_TextIOWrapper__CHUNK_SIZE_set_impl(self, value);
+    return_value = _io_TextIOWrapper__CHUNK_SIZE_set_impl((textio *)self, value);
     Py_END_CRITICAL_SECTION();
 
     return return_value;
 }
-/*[clinic end generated code: output=6f78a6d99fad4893 input=a9049054013a1b77]*/
+/*[clinic end generated code: output=30404271a1151056 input=a9049054013a1b77]*/
