@@ -31,3 +31,13 @@ static const char uninitialized[] = "uninitialized";
 #define UNINITIALIZED_SIZE ((Py_ssize_t)236892191)
 /* Marker to check that integer value was set. */
 #define UNINITIALIZED_INT (63256717)
+/*
+ * Marker to indicate that a NULL parameter would not be allowed
+ * at runtime but that the test interface will check that it is
+ * indeed the case.
+ *
+ * Use this macro only if passing NULL to the C API would raise
+ * a catchable exception (and not a fatal exception that would
+ * crash the interpreter).
+ */
+ #define NULL_WOULD_RAISE(NAME)    NAME
