@@ -3,13 +3,12 @@
 #include "pycore_gc.h"            // PyGC_Head
 #include "pycore_hashtable.h"     // _Py_hashtable_t
 #include "pycore_initconfig.h"    // _PyStatus_NO_MEMORY()
+#include "pycore_interpframe.h"   // _PyInterpreterFrame
+#include "pycore_lock.h"          // PyMutex_LockFlags()
 #include "pycore_object.h"        // _PyType_PreHeaderSize()
 #include "pycore_pymem.h"         // _Py_tracemalloc_config
 #include "pycore_runtime.h"       // _Py_ID()
 #include "pycore_traceback.h"     // _Py_DumpASCII()
-#include <pycore_frame.h>
-
-#include "frameobject.h"          // _PyInterpreterFrame_GetLine
 
 #include <stdlib.h>               // malloc()
 
@@ -20,6 +19,8 @@ _Py_DECLARE_STR(anon_unknown, "<unknown>");
 /* Forward declaration */
 static void* raw_malloc(size_t size);
 static void raw_free(void *ptr);
+static int _PyTraceMalloc_TraceRef(PyObject *op, PyRefTracerEvent event,
+                                   void* Py_UNUSED(ignore));
 
 #ifdef Py_DEBUG
 #  define TRACE_DEBUG
@@ -31,18 +32,12 @@ static void raw_free(void *ptr);
 #define allocators _PyRuntime.tracemalloc.allocators
 
 
-#if defined(TRACE_RAW_MALLOC)
 /* This lock is needed because tracemalloc_free() is called without
    the GIL held from PyMem_RawFree(). It cannot acquire the lock because it
    would introduce a deadlock in _PyThreadState_DeleteCurrent(). */
-#  define tables_lock _PyRuntime.tracemalloc.tables_lock
-#  define TABLES_LOCK() PyThread_acquire_lock(tables_lock, 1)
-#  define TABLES_UNLOCK() PyThread_release_lock(tables_lock)
-#else
-   /* variables are protected by the GIL */
-#  define TABLES_LOCK()
-#  define TABLES_UNLOCK()
-#endif
+#define tables_lock _PyRuntime.tracemalloc.tables_lock
+#define TABLES_LOCK() PyMutex_LockFlags(&tables_lock, _Py_LOCK_DONT_DETACH)
+#define TABLES_UNLOCK() PyMutex_Unlock(&tables_lock)
 
 
 #define DEFAULT_DOMAIN 0
@@ -53,10 +48,7 @@ typedef struct tracemalloc_traceback traceback_t;
 #define TRACEBACK_SIZE(NFRAME) \
         (sizeof(traceback_t) + sizeof(frame_t) * (NFRAME - 1))
 
-/* The maximum number of frames is either:
- - The maximum number of frames we can store in `traceback_t.nframe`
- - The maximum memory size_t we can allocate */
-static const unsigned long MAX_NFRAME = Py_MIN(UINT16_MAX, ((SIZE_MAX - sizeof(traceback_t)) / sizeof(frame_t) + 1));
+static const int MAX_NFRAME = UINT16_MAX;
 
 
 #define tracemalloc_empty_traceback _PyRuntime.tracemalloc.empty_traceback
@@ -96,9 +88,6 @@ tracemalloc_error(const char *format, ...)
 #endif
 
 
-#if defined(TRACE_RAW_MALLOC)
-#define REENTRANT_THREADLOCAL
-
 #define tracemalloc_reentrant_key _PyRuntime.tracemalloc.reentrant_key
 
 /* Any non-NULL pointer can be used */
@@ -107,16 +96,16 @@ tracemalloc_error(const char *format, ...)
 static int
 get_reentrant(void)
 {
-    void *ptr;
-
     assert(PyThread_tss_is_created(&tracemalloc_reentrant_key));
-    ptr = PyThread_tss_get(&tracemalloc_reentrant_key);
+
+    void *ptr = PyThread_tss_get(&tracemalloc_reentrant_key);
     if (ptr != NULL) {
         assert(ptr == REENTRANT);
         return 1;
     }
-    else
+    else {
         return 0;
+    }
 }
 
 static void
@@ -135,25 +124,6 @@ set_reentrant(int reentrant)
     }
 }
 
-#else
-
-/* TRACE_RAW_MALLOC not defined: variable protected by the GIL */
-static int tracemalloc_reentrant = 0;
-
-static int
-get_reentrant(void)
-{
-    return tracemalloc_reentrant;
-}
-
-static void
-set_reentrant(int reentrant)
-{
-    assert(reentrant != tracemalloc_reentrant);
-    tracemalloc_reentrant = reentrant;
-}
-#endif
-
 
 static Py_uhash_t
 hashtable_hash_pyobject(const void *key)
@@ -251,16 +221,16 @@ hashtable_compare_traceback(const void *key1, const void *key2)
 static void
 tracemalloc_get_frame(_PyInterpreterFrame *pyframe, frame_t *frame)
 {
-    assert(PyCode_Check(pyframe->f_executable));
+    assert(PyStackRef_CodeCheck(pyframe->f_executable));
     frame->filename = &_Py_STR(anon_unknown);
+
     int lineno = PyUnstable_InterpreterFrame_GetLine(pyframe);
     if (lineno < 0) {
         lineno = 0;
     }
     frame->lineno = (unsigned int)lineno;
 
-    PyObject *filename = filename = ((PyCodeObject *)pyframe->f_executable)->co_filename;
-
+    PyObject *filename = filename = _PyFrame_GetCode(pyframe)->co_filename;
     if (filename == NULL) {
 #ifdef TRACE_DEBUG
         tracemalloc_error("failed to get the filename of the code object");
@@ -274,14 +244,6 @@ tracemalloc_get_frame(_PyInterpreterFrame *pyframe, frame_t *frame)
 #endif
         return;
     }
-    if (!PyUnicode_IS_READY(filename)) {
-        /* Don't make a Unicode string ready to avoid reentrant calls
-           to tracemalloc_malloc() or tracemalloc_realloc() */
-#ifdef TRACE_DEBUG
-        tracemalloc_error("filename is not a ready unicode string");
-#endif
-        return;
-    }
 
     /* intern the filename */
     _Py_hashtable_entry_t *entry;
@@ -310,7 +272,7 @@ tracemalloc_get_frame(_PyInterpreterFrame *pyframe, frame_t *frame)
 static Py_uhash_t
 traceback_hash(traceback_t *traceback)
 {
-    /* code based on tuplehash() of Objects/tupleobject.c */
+    /* code based on tuple_hash() of Objects/tupleobject.c */
     Py_uhash_t x, y;  /* Unsigned for defined overflow behavior. */
     int len = traceback->nframe;
     Py_uhash_t mult = PyHASH_MULTIPLIER;
@@ -336,13 +298,8 @@ traceback_hash(traceback_t *traceback)
 static void
 traceback_get_frames(traceback_t *traceback)
 {
-    PyThreadState *tstate = PyGILState_GetThisThreadState();
-    if (tstate == NULL) {
-#ifdef TRACE_DEBUG
-        tracemalloc_error("failed to get the current thread state");
-#endif
-        return;
-    }
+    PyThreadState *tstate = _PyThreadState_GET();
+    assert(tstate != NULL);
 
     _PyInterpreterFrame *pyframe = _PyThreadState_GetFrame(tstate);
     while (pyframe) {
@@ -365,7 +322,7 @@ traceback_new(void)
     traceback_t *traceback;
     _Py_hashtable_entry_t *entry;
 
-    assert(PyGILState_Check());
+    _Py_AssertHoldsTstate();
 
     /* get frames */
     traceback = tracemalloc_traceback;
@@ -418,13 +375,21 @@ tracemalloc_create_traces_table(void)
 }
 
 
+static void
+tracemalloc_destroy_domain(void *value)
+{
+    _Py_hashtable_t *ht = (_Py_hashtable_t*)value;
+    _Py_hashtable_destroy(ht);
+}
+
+
 static _Py_hashtable_t*
 tracemalloc_create_domains_table(void)
 {
     return hashtable_new(hashtable_hash_uint,
                          _Py_hashtable_compare_direct,
                          NULL,
-                         (_Py_hashtable_destroy_func)_Py_hashtable_destroy);
+                         tracemalloc_destroy_domain);
 }
 
 
@@ -441,7 +406,7 @@ tracemalloc_get_traces_table(unsigned int domain)
 
 
 static void
-tracemalloc_remove_trace(unsigned int domain, uintptr_t ptr)
+tracemalloc_remove_trace_unlocked(unsigned int domain, uintptr_t ptr)
 {
     assert(tracemalloc_config.tracing);
 
@@ -460,12 +425,12 @@ tracemalloc_remove_trace(unsigned int domain, uintptr_t ptr)
 }
 
 #define REMOVE_TRACE(ptr) \
-            tracemalloc_remove_trace(DEFAULT_DOMAIN, (uintptr_t)(ptr))
+    tracemalloc_remove_trace_unlocked(DEFAULT_DOMAIN, (uintptr_t)(ptr))
 
 
 static int
-tracemalloc_add_trace(unsigned int domain, uintptr_t ptr,
-                      size_t size)
+tracemalloc_add_trace_unlocked(unsigned int domain, uintptr_t ptr,
+                               size_t size)
 {
     assert(tracemalloc_config.tracing);
 
@@ -520,29 +485,54 @@ tracemalloc_add_trace(unsigned int domain, uintptr_t ptr,
 }
 
 #define ADD_TRACE(ptr, size) \
-            tracemalloc_add_trace(DEFAULT_DOMAIN, (uintptr_t)(ptr), size)
+    tracemalloc_add_trace_unlocked(DEFAULT_DOMAIN, (uintptr_t)(ptr), size)
 
 
 static void*
-tracemalloc_alloc(int use_calloc, void *ctx, size_t nelem, size_t elsize)
+tracemalloc_alloc(int need_gil, int use_calloc,
+                  void *ctx, size_t nelem, size_t elsize)
 {
-    PyMemAllocatorEx *alloc = (PyMemAllocatorEx *)ctx;
-    void *ptr;
-
     assert(elsize == 0 || nelem <= SIZE_MAX / elsize);
 
-    if (use_calloc)
-        ptr = alloc->calloc(alloc->ctx, nelem, elsize);
-    else
-        ptr = alloc->malloc(alloc->ctx, nelem * elsize);
-    if (ptr == NULL)
-        return NULL;
+    int reentrant = get_reentrant();
 
+    // Ignore reentrant call.
+    //
+    // For example, PyObjet_Malloc() calls
+    // PyMem_Malloc() for allocations larger than 512 bytes: don't trace the
+    // same memory allocation twice.
+    //
+    // If reentrant calls are not ignored, PyGILState_Ensure() can call
+    // PyMem_RawMalloc() which would call PyGILState_Ensure() again in a loop.
+    if (!reentrant) {
+        set_reentrant(1);
+    }
+
+    PyMemAllocatorEx *alloc = (PyMemAllocatorEx *)ctx;
+    void *ptr;
+    if (use_calloc) {
+        ptr = alloc->calloc(alloc->ctx, nelem, elsize);
+    }
+    else {
+        ptr = alloc->malloc(alloc->ctx, nelem * elsize);
+    }
+
+    if (ptr == NULL) {
+        goto done;
+    }
+    if (reentrant) {
+        goto done;
+    }
+
+    PyGILState_STATE gil_state;
+    if (need_gil) {
+        gil_state = PyGILState_Ensure();
+    }
     TABLES_LOCK();
 
     if (tracemalloc_config.tracing) {
         if (ADD_TRACE(ptr, nelem * elsize) < 0) {
-            /* Failed to allocate a trace for the new memory block */
+            // Failed to allocate a trace for the new memory block
             alloc->free(alloc->ctx, ptr);
             ptr = NULL;
         }
@@ -550,60 +540,93 @@ tracemalloc_alloc(int use_calloc, void *ctx, size_t nelem, size_t elsize)
     // else: gh-128679: tracemalloc.stop() was called by another thread
 
     TABLES_UNLOCK();
+    if (need_gil) {
+        PyGILState_Release(gil_state);
+    }
+
+done:
+    if (!reentrant) {
+        set_reentrant(0);
+    }
     return ptr;
 }
 
 
 static void*
-tracemalloc_realloc(void *ctx, void *ptr, size_t new_size)
+tracemalloc_realloc(int need_gil, void *ctx, void *ptr, size_t new_size)
 {
+    int reentrant = get_reentrant();
+
+    // Ignore reentrant call. PyObjet_Realloc() calls PyMem_Realloc() for
+    // allocations larger than 512 bytes: don't trace the same memory block
+    // twice.
+    if (!reentrant) {
+        set_reentrant(1);
+    }
+
     PyMemAllocatorEx *alloc = (PyMemAllocatorEx *)ctx;
-    void *ptr2;
+    void *ptr2 = alloc->realloc(alloc->ctx, ptr, new_size);
 
-    ptr2 = alloc->realloc(alloc->ctx, ptr, new_size);
-    if (ptr2 == NULL)
-        return NULL;
-
-    TABLES_LOCK();
-    if (!tracemalloc_config.tracing) {
-        // gh-128679: tracemalloc.stop() was called by another thread
+    if (ptr2 == NULL) {
+        goto done;
+    }
+    if (reentrant) {
         goto done;
     }
 
-    if (ptr != NULL) {
-        /* an existing memory block has been resized */
+    PyGILState_STATE gil_state;
+    if (need_gil) {
+        gil_state = PyGILState_Ensure();
+    }
+    TABLES_LOCK();
 
-        /* tracemalloc_add_trace() updates the trace if there is already
-           a trace at address ptr2 */
+    if (!tracemalloc_config.tracing) {
+        // gh-128679: tracemalloc.stop() was called by another thread
+        goto unlock;
+    }
+
+    if (ptr != NULL) {
+        // An existing memory block has been resized
+
+        // tracemalloc_add_trace_unlocked() updates the trace if there is
+        // already a trace at address ptr2.
         if (ptr2 != ptr) {
             REMOVE_TRACE(ptr);
         }
 
         if (ADD_TRACE(ptr2, new_size) < 0) {
-            /* Memory allocation failed. The error cannot be reported to
-               the caller, because realloc() may already have shrunk the
-               memory block and so removed bytes.
-
-               This case is very unlikely: a hash entry has just been
-               released, so the hash table should have at least one free entry.
-
-               The GIL and the table lock ensures that only one thread is
-               allocating memory. */
+            // Memory allocation failed. The error cannot be reported to the
+            // caller, because realloc() already have shrunk the memory block
+            // and so removed bytes.
+            //
+            // This case is very unlikely: a hash entry has just been released,
+            // so the hash table should have at least one free entry.
+            //
+            // The GIL and the table lock ensures that only one thread is
+            // allocating memory.
             Py_FatalError("tracemalloc_realloc() failed to allocate a trace");
         }
     }
     else {
-        /* new allocation */
+        // New allocation
 
         if (ADD_TRACE(ptr2, new_size) < 0) {
-            /* Failed to allocate a trace for the new memory block */
+            // Failed to allocate a trace for the new memory block
             alloc->free(alloc->ctx, ptr2);
             ptr2 = NULL;
         }
     }
 
-done:
+unlock:
     TABLES_UNLOCK();
+    if (need_gil) {
+        PyGILState_Release(gil_state);
+    }
+
+done:
+    if (!reentrant) {
+        set_reentrant(0);
+    }
     return ptr2;
 }
 
@@ -611,16 +634,17 @@ done:
 static void
 tracemalloc_free(void *ctx, void *ptr)
 {
-    PyMemAllocatorEx *alloc = (PyMemAllocatorEx *)ctx;
-
-    if (ptr == NULL)
+    if (ptr == NULL) {
         return;
+    }
 
-     /* GIL cannot be locked in PyMem_RawFree() because it would introduce
-        a deadlock in _PyThreadState_DeleteCurrent(). */
-
+    PyMemAllocatorEx *alloc = (PyMemAllocatorEx *)ctx;
     alloc->free(alloc->ctx, ptr);
 
+    if (get_reentrant()) {
+        return;
+    }
+
     TABLES_LOCK();
 
     if (tracemalloc_config.tracing) {
@@ -632,158 +656,46 @@ tracemalloc_free(void *ctx, void *ptr)
 }
 
 
-static void*
-tracemalloc_alloc_gil(int use_calloc, void *ctx, size_t nelem, size_t elsize)
-{
-    void *ptr;
-
-    if (get_reentrant()) {
-        PyMemAllocatorEx *alloc = (PyMemAllocatorEx *)ctx;
-        if (use_calloc)
-            return alloc->calloc(alloc->ctx, nelem, elsize);
-        else
-            return alloc->malloc(alloc->ctx, nelem * elsize);
-    }
-
-    /* Ignore reentrant call. PyObjet_Malloc() calls PyMem_Malloc() for
-       allocations larger than 512 bytes, don't trace the same memory
-       allocation twice. */
-    set_reentrant(1);
-
-    ptr = tracemalloc_alloc(use_calloc, ctx, nelem, elsize);
-
-    set_reentrant(0);
-    return ptr;
-}
-
-
 static void*
 tracemalloc_malloc_gil(void *ctx, size_t size)
 {
-    return tracemalloc_alloc_gil(0, ctx, 1, size);
+    return tracemalloc_alloc(0, 0, ctx, 1, size);
 }
 
 
 static void*
 tracemalloc_calloc_gil(void *ctx, size_t nelem, size_t elsize)
 {
-    return tracemalloc_alloc_gil(1, ctx, nelem, elsize);
+    return tracemalloc_alloc(0, 1, ctx, nelem, elsize);
 }
 
 
 static void*
 tracemalloc_realloc_gil(void *ctx, void *ptr, size_t new_size)
 {
-    void *ptr2;
-
-    if (get_reentrant()) {
-        /* Reentrant call to PyMem_Realloc() and PyMem_RawRealloc().
-           Example: PyMem_RawRealloc() is called internally by pymalloc
-           (_PyObject_Malloc() and  _PyObject_Realloc()) to allocate a new
-           arena (new_arena()). */
-        PyMemAllocatorEx *alloc = (PyMemAllocatorEx *)ctx;
-
-        ptr2 = alloc->realloc(alloc->ctx, ptr, new_size);
-        if (ptr2 != NULL && ptr != NULL) {
-            TABLES_LOCK();
-            if (tracemalloc_config.tracing) {
-                REMOVE_TRACE(ptr);
-            }
-            TABLES_UNLOCK();
-        }
-        return ptr2;
-    }
-
-    /* Ignore reentrant call. PyObjet_Realloc() calls PyMem_Realloc() for
-       allocations larger than 512 bytes. Don't trace the same memory
-       allocation twice. */
-    set_reentrant(1);
-
-    ptr2 = tracemalloc_realloc(ctx, ptr, new_size);
-
-    set_reentrant(0);
-    return ptr2;
-}
-
-
-#ifdef TRACE_RAW_MALLOC
-static void*
-tracemalloc_raw_alloc(int use_calloc, void *ctx, size_t nelem, size_t elsize)
-{
-    PyGILState_STATE gil_state;
-    void *ptr;
-
-    if (get_reentrant()) {
-        PyMemAllocatorEx *alloc = (PyMemAllocatorEx *)ctx;
-        if (use_calloc)
-            return alloc->calloc(alloc->ctx, nelem, elsize);
-        else
-            return alloc->malloc(alloc->ctx, nelem * elsize);
-    }
-
-    /* Ignore reentrant call. PyGILState_Ensure() may call PyMem_RawMalloc()
-       indirectly which would call PyGILState_Ensure() if reentrant are not
-       disabled. */
-    set_reentrant(1);
-
-    gil_state = PyGILState_Ensure();
-    ptr = tracemalloc_alloc(use_calloc, ctx, nelem, elsize);
-    PyGILState_Release(gil_state);
-
-    set_reentrant(0);
-    return ptr;
+    return tracemalloc_realloc(0, ctx, ptr, new_size);
 }
 
 
 static void*
 tracemalloc_raw_malloc(void *ctx, size_t size)
 {
-    return tracemalloc_raw_alloc(0, ctx, 1, size);
+    return tracemalloc_alloc(1, 0, ctx, 1, size);
 }
 
 
 static void*
 tracemalloc_raw_calloc(void *ctx, size_t nelem, size_t elsize)
 {
-    return tracemalloc_raw_alloc(1, ctx, nelem, elsize);
+    return tracemalloc_alloc(1, 1, ctx, nelem, elsize);
 }
 
 
 static void*
 tracemalloc_raw_realloc(void *ctx, void *ptr, size_t new_size)
 {
-    PyGILState_STATE gil_state;
-    void *ptr2;
-
-    if (get_reentrant()) {
-        /* Reentrant call to PyMem_RawRealloc(). */
-        PyMemAllocatorEx *alloc = (PyMemAllocatorEx *)ctx;
-
-        ptr2 = alloc->realloc(alloc->ctx, ptr, new_size);
-
-        if (ptr2 != NULL && ptr != NULL) {
-            TABLES_LOCK();
-            if (tracemalloc_config.tracing) {
-                REMOVE_TRACE(ptr);
-            }
-            TABLES_UNLOCK();
-        }
-        return ptr2;
-    }
-
-    /* Ignore reentrant call. PyGILState_Ensure() may call PyMem_RawMalloc()
-       indirectly which would call PyGILState_Ensure() if reentrant calls are
-       not disabled. */
-    set_reentrant(1);
-
-    gil_state = PyGILState_Ensure();
-    ptr2 = tracemalloc_realloc(ctx, ptr, new_size);
-    PyGILState_Release(gil_state);
-
-    set_reentrant(0);
-    return ptr2;
+    return tracemalloc_realloc(1, ctx, ptr, new_size);
 }
-#endif   /* TRACE_RAW_MALLOC */
 
 
 static void
@@ -794,24 +706,22 @@ tracemalloc_clear_filename(void *value)
 }
 
 
-/* reentrant flag must be set to call this function and GIL must be held */
 static void
 tracemalloc_clear_traces_unlocked(void)
 {
-    set_reentrant(1);
+    // Clearing tracemalloc_filenames requires the GIL to call Py_DECREF()
+    _Py_AssertHoldsTstate();
 
-    /* The GIL protects variables against concurrent access */
-    assert(PyGILState_Check());
+    set_reentrant(1);
 
     _Py_hashtable_clear(tracemalloc_traces);
     _Py_hashtable_clear(tracemalloc_domains);
+    _Py_hashtable_clear(tracemalloc_tracebacks);
+    _Py_hashtable_clear(tracemalloc_filenames);
+
     tracemalloc_traced_memory = 0;
     tracemalloc_peak_traced_memory = 0;
 
-    _Py_hashtable_clear(tracemalloc_tracebacks);
-
-    _Py_hashtable_clear(tracemalloc_filenames);
-
     set_reentrant(0);
 }
 
@@ -823,20 +733,9 @@ _PyTraceMalloc_Init(void)
 
     PyMem_GetAllocator(PYMEM_DOMAIN_RAW, &allocators.raw);
 
-#ifdef REENTRANT_THREADLOCAL
     if (PyThread_tss_create(&tracemalloc_reentrant_key) != 0) {
         return _PyStatus_NO_MEMORY();
     }
-#endif
-
-#if defined(TRACE_RAW_MALLOC)
-    if (tables_lock == NULL) {
-        tables_lock = PyThread_allocate_lock();
-        if (tables_lock == NULL) {
-            return _PyStatus_NO_MEMORY();
-        }
-    }
-#endif
 
     tracemalloc_filenames = hashtable_new(hashtable_hash_pyobject,
                                           hashtable_compare_unicode,
@@ -882,45 +781,29 @@ tracemalloc_deinit(void)
     _Py_hashtable_destroy(tracemalloc_tracebacks);
     _Py_hashtable_destroy(tracemalloc_filenames);
 
-#if defined(TRACE_RAW_MALLOC)
-    if (tables_lock != NULL) {
-        PyThread_free_lock(tables_lock);
-        tables_lock = NULL;
-    }
-#endif
-
-#ifdef REENTRANT_THREADLOCAL
     PyThread_tss_delete(&tracemalloc_reentrant_key);
-#endif
 }
 
 
 int
 _PyTraceMalloc_Start(int max_nframe)
 {
-    PyMemAllocatorEx alloc;
-    size_t size;
-
-    if (max_nframe < 1 || (unsigned long) max_nframe > MAX_NFRAME) {
+    if (max_nframe < 1 || max_nframe > MAX_NFRAME) {
         PyErr_Format(PyExc_ValueError,
-                     "the number of frames must be in range [1; %lu]",
+                     "the number of frames must be in range [1; %i]",
                      MAX_NFRAME);
         return -1;
     }
 
-    if (PyRefTracer_SetTracer(_PyTraceMalloc_TraceRef, NULL) < 0) {
-        return -1;
-    }
-
-    if (tracemalloc_config.tracing) {
-        /* hook already installed: do nothing */
+    if (_PyTraceMalloc_IsTracing()) {
+        /* hooks already installed: do nothing */
         return 0;
     }
 
     tracemalloc_config.max_nframe = max_nframe;
 
     /* allocate a buffer to store a new traceback */
-    size = TRACEBACK_SIZE(max_nframe);
+    size_t size = TRACEBACK_SIZE(max_nframe);
     assert(tracemalloc_traceback == NULL);
     tracemalloc_traceback = raw_malloc(size);
     if (tracemalloc_traceback == NULL) {
@@ -928,7 +811,7 @@ _PyTraceMalloc_Start(int max_nframe)
         return -1;
     }
 
-#ifdef TRACE_RAW_MALLOC
+    PyMemAllocatorEx alloc;
     alloc.malloc = tracemalloc_raw_malloc;
     alloc.calloc = tracemalloc_raw_calloc;
     alloc.realloc = tracemalloc_raw_realloc;
@@ -937,7 +820,6 @@ _PyTraceMalloc_Start(int max_nframe)
     alloc.ctx = &allocators.raw;
     PyMem_GetAllocator(PYMEM_DOMAIN_RAW, &allocators.raw);
     PyMem_SetAllocator(PYMEM_DOMAIN_RAW, &alloc);
-#endif
 
     alloc.malloc = tracemalloc_malloc_gil;
     alloc.calloc = tracemalloc_calloc_gil;
@@ -952,8 +834,14 @@ _PyTraceMalloc_Start(int max_nframe)
     PyMem_GetAllocator(PYMEM_DOMAIN_OBJ, &allocators.obj);
     PyMem_SetAllocator(PYMEM_DOMAIN_OBJ, &alloc);
 
+    if (PyRefTracer_SetTracer(_PyTraceMalloc_TraceRef, NULL) < 0) {
+        return -1;
+    }
+
     /* everything is ready: start tracing Python memory allocations */
+    TABLES_LOCK();
     tracemalloc_config.tracing = 1;
+    TABLES_UNLOCK();
 
     return 0;
 }
@@ -962,8 +850,6 @@ _PyTraceMalloc_Start(int max_nframe)
 void
 _PyTraceMalloc_Stop(void)
 {
-    // Lock to synchronize with tracemalloc_free() which checks
-    // 'tracing' while holding the lock.
     TABLES_LOCK();
 
     if (!tracemalloc_config.tracing) {
@@ -974,9 +860,7 @@ _PyTraceMalloc_Stop(void)
     tracemalloc_config.tracing = 0;
 
     /* unregister the hook on memory allocators */
-#ifdef TRACE_RAW_MALLOC
     PyMem_SetAllocator(PYMEM_DOMAIN_RAW, &allocators.raw);
-#endif
     PyMem_SetAllocator(PYMEM_DOMAIN_MEM, &allocators.mem);
     PyMem_SetAllocator(PYMEM_DOMAIN_OBJ, &allocators.obj);
 
@@ -997,15 +881,16 @@ done:
 static PyObject*
 frame_to_pyobject(frame_t *frame)
 {
-    PyObject *frame_obj, *lineno_obj;
+    assert(get_reentrant());
 
-    frame_obj = PyTuple_New(2);
-    if (frame_obj == NULL)
+    PyObject *frame_obj = PyTuple_New(2);
+    if (frame_obj == NULL) {
         return NULL;
+    }
 
     PyTuple_SET_ITEM(frame_obj, 0, Py_NewRef(frame->filename));
 
-    lineno_obj = PyLong_FromUnsignedLong(frame->lineno);
+    PyObject *lineno_obj = PyLong_FromUnsignedLong(frame->lineno);
     if (lineno_obj == NULL) {
         Py_DECREF(frame_obj);
         return NULL;
@@ -1020,7 +905,6 @@ static PyObject*
 traceback_to_pyobject(traceback_t *traceback, _Py_hashtable_t *intern_table)
 {
     PyObject *frames;
-
     if (intern_table != NULL) {
         frames = _Py_hashtable_get(intern_table, (const void *)traceback);
         if (frames) {
@@ -1029,8 +913,9 @@ traceback_to_pyobject(traceback_t *traceback, _Py_hashtable_t *intern_table)
     }
 
     frames = PyTuple_New(traceback->nframe);
-    if (frames == NULL)
+    if (frames == NULL) {
         return NULL;
+    }
 
     for (int i=0; i < traceback->nframe; i++) {
         PyObject *frame = frame_to_pyobject(&traceback->frames[i]);
@@ -1058,14 +943,14 @@ static PyObject*
 trace_to_pyobject(unsigned int domain, const trace_t *trace,
                   _Py_hashtable_t *intern_tracebacks)
 {
-    PyObject *trace_obj = NULL;
-    PyObject *obj;
+    assert(get_reentrant());
 
-    trace_obj = PyTuple_New(4);
-    if (trace_obj == NULL)
+    PyObject *trace_obj = PyTuple_New(4);
+    if (trace_obj == NULL) {
         return NULL;
+    }
 
-    obj = PyLong_FromSize_t(domain);
+    PyObject *obj = PyLong_FromSize_t(domain);
     if (obj == NULL) {
         Py_DECREF(trace_obj);
         return NULL;
@@ -1112,7 +997,6 @@ tracemalloc_copy_trace(_Py_hashtable_t *traces,
                        void *user_data)
 {
     _Py_hashtable_t *traces2 = (_Py_hashtable_t *)user_data;
-
     trace_t *trace = (trace_t *)value;
 
     trace_t *trace2 = raw_malloc(sizeof(trace_t));
@@ -1153,7 +1037,6 @@ tracemalloc_copy_domain(_Py_hashtable_t *domains,
                         void *user_data)
 {
     _Py_hashtable_t *domains2 = (_Py_hashtable_t *)user_data;
-
     unsigned int domain = (unsigned int)FROM_PTR(key);
     _Py_hashtable_t *traces = (_Py_hashtable_t *)value;
 
@@ -1194,7 +1077,6 @@ tracemalloc_get_traces_fill(_Py_hashtable_t *traces,
                             void *user_data)
 {
     get_traces_t *get_traces = user_data;
-
     const trace_t *trace = (const trace_t *)value;
 
     PyObject *tuple = trace_to_pyobject(get_traces->domain, trace,
@@ -1208,7 +1090,6 @@ tracemalloc_get_traces_fill(_Py_hashtable_t *traces,
     if (res < 0) {
         return 1;
     }
-
     return 0;
 }
 
@@ -1219,7 +1100,6 @@ tracemalloc_get_traces_domain(_Py_hashtable_t *domains,
                               void *user_data)
 {
     get_traces_t *get_traces = user_data;
-
     unsigned int domain = (unsigned int)FROM_PTR(key);
     _Py_hashtable_t *traces = (_Py_hashtable_t *)value;
 
@@ -1246,14 +1126,14 @@ tracemalloc_get_traceback_unlocked(unsigned int domain, uintptr_t ptr)
     }
 
     _Py_hashtable_t *traces = tracemalloc_get_traces_table(domain);
-    trace_t *trace = NULL;
-    if (traces) {
-        trace = _Py_hashtable_get(traces, TO_PTR(ptr));
-    }
-    if (!trace) {
+    if (!traces) {
         return NULL;
     }
 
+    trace_t *trace = _Py_hashtable_get(traces, TO_PTR(ptr));
+    if (!trace) {
+        return NULL;
+    }
     return trace->traceback;
 }
 
@@ -1275,31 +1155,28 @@ _PyMem_DumpFrame(int fd, frame_t * frame)
 void
 _PyMem_DumpTraceback(int fd, const void *ptr)
 {
-    traceback_t *traceback;
-    int i;
-
     TABLES_LOCK();
-
-    if (tracemalloc_config.tracing) {
-        traceback = tracemalloc_get_traceback_unlocked(DEFAULT_DOMAIN,
-                                                       (uintptr_t)ptr);
-    }
-    else {
-        traceback = NULL;
+    if (!tracemalloc_config.tracing) {
         PUTS(fd, "Enable tracemalloc to get the memory block "
                  "allocation traceback\n\n");
+        goto done;
     }
 
-    TABLES_UNLOCK();
-
-    if (traceback == NULL)
-        return;
+    traceback_t *traceback;
+    traceback = tracemalloc_get_traceback_unlocked(DEFAULT_DOMAIN,
+                                                   (uintptr_t)ptr);
+    if (traceback == NULL) {
+        goto done;
+    }
 
     PUTS(fd, "Memory block allocated at (most recent call first):\n");
-    for (i=0; i < traceback->nframe; i++) {
+    for (int i=0; i < traceback->nframe; i++) {
         _PyMem_DumpFrame(fd, &traceback->frames[i]);
     }
     PUTS(fd, "\n");
+
+done:
+    TABLES_UNLOCK();
 }
 
 #undef PUTS
@@ -1321,27 +1198,18 @@ PyTraceMalloc_Track(unsigned int domain, uintptr_t ptr,
                     size_t size)
 {
     PyGILState_STATE gil_state = PyGILState_Ensure();
-    int result;
-
-    // gh-129185: Check before TABLES_LOCK() to support calls after
-    // _PyTraceMalloc_Fini().
-    if (!tracemalloc_config.tracing) {
-        result = -2;
-        goto done;
-    }
-
     TABLES_LOCK();
 
+    int result;
     if (tracemalloc_config.tracing) {
-        result = tracemalloc_add_trace(domain, ptr, size);
+        result = tracemalloc_add_trace_unlocked(domain, ptr, size);
     }
     else {
-        // gh-128679: tracemalloc.stop() was called by another thread
+        /* tracemalloc is not tracing: do nothing */
         result = -2;
     }
 
     TABLES_UNLOCK();
-done:
     PyGILState_Release(gil_state);
     return result;
 }
@@ -1350,21 +1218,11 @@ done:
 int
 PyTraceMalloc_Untrack(unsigned int domain, uintptr_t ptr)
 {
-    // Need the GIL to prevent races on the first 'tracing' test
-    PyGILState_STATE gil_state = PyGILState_Ensure();
-    int result;
-
-    // gh-129185: Check before TABLES_LOCK() to support calls after
-    // _PyTraceMalloc_Fini()
-    if (!tracemalloc_config.tracing) {
-        result = -2;
-        goto done;
-    }
-
     TABLES_LOCK();
 
+    int result;
     if (tracemalloc_config.tracing) {
-        tracemalloc_remove_trace(domain, ptr);
+        tracemalloc_remove_trace_unlocked(domain, ptr);
         result = 0;
     }
     else {
@@ -1373,8 +1231,6 @@ PyTraceMalloc_Untrack(unsigned int domain, uintptr_t ptr)
     }
 
     TABLES_UNLOCK();
-done:
-    PyGILState_Release(gil_state);
     return result;
 }
 
@@ -1382,7 +1238,7 @@ done:
 void
 _PyTraceMalloc_Fini(void)
 {
-    assert(PyGILState_Check());
+    _Py_AssertHoldsTstate();
     tracemalloc_deinit();
 }
 
@@ -1392,63 +1248,62 @@ _PyTraceMalloc_Fini(void)
 
    Do nothing if tracemalloc is not tracing memory allocations
    or if the object memory block is not already traced. */
-int
-_PyTraceMalloc_TraceRef(PyObject *op, PyRefTracerEvent event, void* Py_UNUSED(ignore))
+static int
+_PyTraceMalloc_TraceRef(PyObject *op, PyRefTracerEvent event,
+                        void* Py_UNUSED(ignore))
 {
     if (event != PyRefTracer_CREATE) {
         return 0;
     }
+    if (get_reentrant()) {
+        return 0;
+    }
 
-    assert(PyGILState_Check());
+    _Py_AssertHoldsTstate();
+    TABLES_LOCK();
 
     if (!tracemalloc_config.tracing) {
-        /* tracemalloc is not tracing: do nothing */
-        return -1;
+        goto done;
     }
 
     PyTypeObject *type = Py_TYPE(op);
     const size_t presize = _PyType_PreHeaderSize(type);
     uintptr_t ptr = (uintptr_t)((char *)op - presize);
 
-    int res = -1;
-
-    TABLES_LOCK();
-
-    if (!tracemalloc_config.tracing) {
-        // gh-128679: tracemalloc.stop() was called by another thread
-        goto done;
-    }
-
     trace_t *trace = _Py_hashtable_get(tracemalloc_traces, TO_PTR(ptr));
     if (trace != NULL) {
         /* update the traceback of the memory block */
         traceback_t *traceback = traceback_new();
         if (traceback != NULL) {
             trace->traceback = traceback;
-            res = 0;
         }
     }
     /* else: cannot track the object, its memory block size is unknown */
 
 done:
     TABLES_UNLOCK();
-
-    return res;
+    return 0;
 }
 
 
 PyObject*
 _PyTraceMalloc_GetTraceback(unsigned int domain, uintptr_t ptr)
 {
-    traceback_t *traceback;
-
     TABLES_LOCK();
-    traceback = tracemalloc_get_traceback_unlocked(domain, ptr);
-    TABLES_UNLOCK();
-    if (traceback == NULL)
-        Py_RETURN_NONE;
 
-    return traceback_to_pyobject(traceback, NULL);
+    traceback_t *traceback = tracemalloc_get_traceback_unlocked(domain, ptr);
+    PyObject *result;
+    if (traceback) {
+        set_reentrant(1);
+        result = traceback_to_pyobject(traceback, NULL);
+        set_reentrant(0);
+    }
+    else {
+        result = Py_NewRef(Py_None);
+    }
+
+    TABLES_UNLOCK();
+    return result;
 }
 
 int
@@ -1473,17 +1328,22 @@ _PyTraceMalloc_ClearTraces(void)
 PyObject *
 _PyTraceMalloc_GetTraces(void)
 {
+    TABLES_LOCK();
+    set_reentrant(1);
+
     get_traces_t get_traces;
     get_traces.domain = DEFAULT_DOMAIN;
     get_traces.traces = NULL;
     get_traces.domains = NULL;
     get_traces.tracebacks = NULL;
     get_traces.list = PyList_New(0);
-    if (get_traces.list == NULL)
-        goto error;
+    if (get_traces.list == NULL) {
+        goto finally;
+    }
 
-    if (!tracemalloc_config.tracing)
-        return get_traces.list;
+    if (!tracemalloc_config.tracing) {
+        goto finally;
+    }
 
     /* the traceback hash table is used temporarily to intern traceback tuple
        of (filename, lineno) tuples */
@@ -1497,24 +1357,17 @@ _PyTraceMalloc_GetTraces(void)
     // Copy all traces so tracemalloc_get_traces_fill() doesn't have to disable
     // temporarily tracemalloc which would impact other threads and so would
     // miss allocations while get_traces() is called.
-    TABLES_LOCK();
     get_traces.traces = tracemalloc_copy_traces(tracemalloc_traces);
-    TABLES_UNLOCK();
-
     if (get_traces.traces == NULL) {
         goto no_memory;
     }
 
-    TABLES_LOCK();
     get_traces.domains = tracemalloc_copy_domains(tracemalloc_domains);
-    TABLES_UNLOCK();
-
     if (get_traces.domains == NULL) {
         goto no_memory;
     }
 
     // Convert traces to a list of tuples
-    set_reentrant(1);
     int err = _Py_hashtable_foreach(get_traces.traces,
                                     tracemalloc_get_traces_fill,
                                     &get_traces);
@@ -1523,20 +1376,22 @@ _PyTraceMalloc_GetTraces(void)
                                     tracemalloc_get_traces_domain,
                                     &get_traces);
     }
-    set_reentrant(0);
-    if (err) {
-        goto error;
-    }
 
+    if (err) {
+        Py_CLEAR(get_traces.list);
+        goto finally;
+    }
     goto finally;
 
 no_memory:
     PyErr_NoMemory();
-
-error:
     Py_CLEAR(get_traces.list);
+    goto finally;
 
 finally:
+    set_reentrant(0);
+    TABLES_UNLOCK();
+
     if (get_traces.tracebacks != NULL) {
         _Py_hashtable_destroy(get_traces.tracebacks);
     }
@@ -1560,19 +1415,20 @@ _PyTraceMalloc_GetObjectTraceback(PyObject *obj)
     return _PyTraceMalloc_GetTraceback(DEFAULT_DOMAIN, ptr);
 }
 
-int _PyTraceMalloc_GetTracebackLimit(void) {
+int _PyTraceMalloc_GetTracebackLimit(void)
+{
     return tracemalloc_config.max_nframe;
 }
 
 size_t
-_PyTraceMalloc_GetMemory(void) {
-
-    size_t size;
-
+_PyTraceMalloc_GetMemory(void)
+{
     TABLES_LOCK();
+    size_t size;
     if (tracemalloc_config.tracing) {
         size = _Py_hashtable_size(tracemalloc_tracebacks);
         size += _Py_hashtable_size(tracemalloc_filenames);
+
         size += _Py_hashtable_size(tracemalloc_traces);
         _Py_hashtable_foreach(tracemalloc_domains,
                               tracemalloc_get_tracemalloc_memory_cb, &size);
@@ -1581,7 +1437,6 @@ _PyTraceMalloc_GetMemory(void) {
         size = 0;
     }
     TABLES_UNLOCK();
-
     return size;
 }
 
@@ -1589,20 +1444,19 @@ _PyTraceMalloc_GetMemory(void) {
 PyObject *
 _PyTraceMalloc_GetTracedMemory(void)
 {
-    Py_ssize_t size, peak_size;
-
     TABLES_LOCK();
+    Py_ssize_t traced, peak;
     if (tracemalloc_config.tracing) {
-        size = tracemalloc_traced_memory;
-        peak_size = tracemalloc_peak_traced_memory;
+        traced = tracemalloc_traced_memory;
+        peak = tracemalloc_peak_traced_memory;
     }
     else {
-        size = 0;
-        peak_size = 0;
+        traced = 0;
+        peak = 0;
     }
     TABLES_UNLOCK();
 
-    return Py_BuildValue("nn", size, peak_size);
+    return Py_BuildValue("nn", traced, peak);
 }
 
 void
